;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; SMW Classic Piranha Plants (sprites 1A & 2A), by imamelia
;;
;; This is a disassembly of sprites 1A and 2A in SMW, the normal and upside-down
;; Classic Piranha Plants.
;;
;; Uses first extra bit: YES
;;
;; If the first extra bit is clear, the Piranha Plant will be rightside-up.  If the first extra
;; bit is set, the Piranha Plant will be upside-down.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;incsrc subroutinedefsx.asm

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; defines and tables
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Speed:			; the Piranha Plant's Y speed for each sprite state (inverted for the upside-down plant)
db $00,$F0,$00,$10		; in the pipe, moving forward, resting at the apex, moving backward
TimeInState:		; the time the plant will spend in each sprite state
db $20,$30,$20,$30	; in the pipe, moving forward, resting at the apex, moving backward

TileYOffset:			; tile Y offset for the graphics routine
db $10,$00,$10,$00,$00,$10,$00,$10	; depending on direction

Tilemap:				; the tiles for the graphics routine
db $CE,$AE,$CE,$AC,$CE,$AE,$CE,$AC	; stem, head frame 1, stem, head frame 2

TileProperties:			; tile properties for the graphics routine
db $5A,$58,$5A,$58,$DA,$D8,$DA,$D8	; stem palette D, head palette C, Y-flip depending on direction,
				; alway GFX page 1 and priority setting 1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "INIT ",pc
JSR PiranhaInit
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; init routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PiranhaInit:

LDA $7FAB10,x	; check the extra bit
AND #$04	; if the extra bit is clear...
LSR		;
LSR		;
STA $1510,x	;
BEQ InitUpPiranha	; skip the part of the init routine strictly for the upside-down plant

;ASL $15F6,x	; These three lines are simply to flip the tiles of the upside-down plant.
;SEC		; I commented them out, because the Piranha Plant doesn't have its own
;ROR $15F6,x	; graphics routine, so I had to write one anyway.
LDA $D8,x	; 
SEC		; offset its Y position
SBC #$10		; by 16 pixels
STA $D8,x	;
LDA $14D4,x	; handle the high byte
SBC #$00		; to prevent overflow
STA $14D4,x	;

InitUpPiranha:	;

LDA $E4,x	;
CLC		;
ADC #$08	; shift the sprite's X position to the right 8 pixels
STA $E4,x		;
DEC $D8,x	; shift the sprite down 1 pixels
LDA $D8,x	;
CMP #$FF		; if there was overflow...
BNE EndInit	;
DEC $14D4,x	; decrement the high byte as well
EndInit:		;
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine wrapper
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

print "MAIN ",pc
PHB
PHK
PLB
JSR PiranhaPlantsMain
PLB
RTL

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; main routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PiranhaPlantsMain:

LDA $1594,x		; if the sprite is in a pipe and the player is near...
BNE NoGFX		; don't draw the sprite

JSR PiranhaPlantGFX		; draw the sprite

NoGFX:			;

JSR SubOffscreenX0		;

LDA $9D			; if sprites are locked...
BNE EndMain		; terminate the main routine right here

JSR SetAnimationFrame	; determine which frame the plant should show

LDA $1594,x		; if the plant is in a pipe...
BNE NoInteraction		; don't let it interact with the player

JSL $01803A		; interact with the player and other sprites

NoInteraction:		;

LDA $C2,x		; use the sprite state
AND #$03		; to determine what the sprite's speed should be
TAY			;
LDA $1540,x		; if the timer for changing states has run out...
BEQ ChangePiranhaState	;

LDA Speed,y		; load the base speed
LDY $1510,x		; check whether the sprite is rightside-up or upside-down
BEQ StoreSpeed		; if upside-down...
EOR #$FF			; flip its speed
INC			;
StoreSpeed:		;
STA $AA,x		; store the speed value to the appropriate sprite table

JSL $01801A		; update sprite Y position without gravity

EndMain:			;
RTS			;

ChangePiranhaState:	;

LDA $C2,x		; sprite state
AND #$03		; 4 possible states, so we need only 2 bits
STA $00			; store to scratch RAM for subsequent use
BNE NoProximityCheck	; if not state 00 (in the pipe), don't check to see if the player is near

JSR SubHorzPos		; get the horizontal distance between the player and the sprite

LDA #$01			;
STA $1594,x		; set the invisibility flag if necessary
LDA $0F			;
CLC			;
ADC #$1B			; if the sprite is within a certain distance...
CMP #$37			;
BCC EndStateChange	; don't change the sprite state

NoProximityCheck:		;

STZ $1594,x		; if the sprite is out of range, clear the invisibility flag
LDY $00			; load the state value from before
LDA TimeInState,y		;
STA $1540,x		; set the time to change state
INC $C2,x		; and increment the sprite state

EndStateChange:		;
RTS

SetAnimationFrame:		;

INC $1570,x		; $1570,x - individual sprite frame counter, in this context
LDA $1570,x		;
LSR #3			; change image every 8 frames
AND #$01		;
STA $1602,x		; set the resulting image
RTS

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; graphics routine
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

PiranhaPlantGFX:		; I made my own graphics routine, since the Piranha Plant uses a shared routine.

JSR GetDrawInfo		; set some variables up for writing to OAM

LDA $1510,x		; direction bit
ASL			; x2
ASL			; x4
STA $03			; set the tilemap index
LDA $1602,x		; current frame
ASL			;
ORA #$01			;
ORA $03			;
STA $03			; set the index for the Y offsets depending on frame and direction

LDA #$01			; 2 tiles to draw
STA $04			;

LDX $03

GFXLoop:			;

LDA $00			;
STA $0300,y		; no X offset

LDA $01			;
CLC			;
ADC TileYOffset,x		; set the tile Y offset depending on direction and which tile is being drawn
STA $0301,y		;

LDA Tilemap,x		; set the tile number
STA $0302,y		;

LDA TileProperties,x	; set the tile properties
STA $0303,y		;

INY #4			; increment Y to get to the next OAM slot
DEX			;
DEC $04			; decrement the tile counter
BPL GFXLoop		; if still positive, there is another tile to draw

LDX $15E9		; sprite index back into X
LDY #$02			; the tiles were 16x16
LDA #$01			; we drew 2 tiles
JSL $81B7B3		;

RTS			;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;; generic subroutines (retyped and uncommented)
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Table1:              db $0C,$1C
Table2:              db $01,$02
Table3:              db $40,$B0
Table6:              db $01,$FF
Table4:              db $30,$C0,$A0,$C0,$A0,$F0,$60,$90,$30,$C0,$A0,$80,$A0,$40,$60,$B0
Table5:              db $01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$FF,$01,$00,$01,$FF

SubHorzPos:

LDY #$00
LDA $94
SEC
SBC $E4,x
STA $0F
LDA $95
SBC $14E0,x
BPL $01
INY
RTS

SubOffscreenX0:
LDA #$00
;BRA SubOffscreenMain
;SubOffscreenX1:
;LDA #$02
;BRA SubOffscreenMain
;SubOffscreenX2:
;LDA #$04
;BRA SubOffscreenMain
;SubOffscreenX3:
;LDA #$06
;BRA SubOffscreenMain
;SubOffscreenX4:
;LDA #$08
;BRA SubOffscreenMain
;SubOffscreenX5:
;LDA #$0A
;BRA SubOffscreenMain
;SubOffscreenX6:
;LDA #$0C
;BRA SubOffscreenMain
;SubOffscreenX7:
;LDA #$0E

SubOffscreenMain:

STA $03

JSR SubIsOffscreen
BEQ Return2

LDA $5B
LSR
BCS VerticalLevel
LDA $D8,x
CLC
ADC #$50
LDA $14D4,x
ADC #$00
CMP #$02
BPL EraseSprite
LDA $167A,x
AND #$04
BNE Return2
LDA $13
AND #$01
ORA $03
STA $01
TAY
LDA $1A
CLC
ADC Table4,y
ROL $00
CMP $E4,x
PHP
LDA $1B
LSR $00
ADC Table5,y
PLP
SBC $14E0,x
STA $00
LSR $01
BCC Label20
EOR #$80
STA $00
Label20:
LDA $00
BPL Return2

EraseSprite:
LDA $14C8,x
CMP #$08
BCC KillSprite
LDY $161A,x
CPY #$FF
BEQ KillSprite
LDA #$00
STA $1938,y
KillSprite:
STZ $14C8,x
Return2:
RTS

VerticalLevel:

LDA $167A,x
AND #$04
BNE Return2
LDA $13
LSR
BCS Return2
AND #$01
STA $01
TAY
LDA $1C
CLC
ADC Table3,y
ROL $00
CMP $D8,x
PHP
LDA $1D
LSR $00
ADC Table6,y
PLP
SBC $14D4,x
STA $00
LDY $02
BEQ Label22
EOR #$80
STA $00
Label22:
LDA $00
BPL Return2
BMI EraseSprite

SubIsOffscreen:
LDA $15A0,x
ORA $186C,x
RTS

GetDrawInfo:

STZ $186C,x
STZ $15A0,x
LDA $E4,x
CMP $1A
LDA $14E0,x
SBC $1B
BEQ OnscreenX
INC $15A0,x
OnscreenX:
LDA $14E0,x
XBA
LDA $E4,x
REP #$20
SEC
SBC $1A
CLC
ADC.w #$0040
CMP #$0180
SEP #$20
ROL A
AND #$01
STA $15C4,x
BNE Invalid

LDY #$00
LDA $1662,x
AND #$20
BEQ OnscreenLoop
INY
OnscreenLoop:
LDA $D8,x
CLC
ADC Table1,y
PHP
CMP $1C
ROL $00
PLP
LDA $14D4,x
ADC #$00
LSR $00
SBC $1D
BEQ OnscreenY
LDA $186C,x
ORA Table2,y
STA $186C,x
OnscreenY:
DEY
BPL OnscreenLoop
LDY $15EA,x
LDA $E4,x
SEC
SBC $1A
STA $00
LDA $D8,x
SEC
SBC $1C
STA $01
RTS

Invalid:
PLA
PLA
RTS










